'' =================================================================================================
''
''   File....... jm_dual_pwm_ez.spin
''   Purpose.... Dual-channel pwm driver
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2010-2022 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon.mcphalen@gmail.com
''   Started....
''   Updated.... 21 APR 2018
''
'' =================================================================================================


con

' max freq based on 5MHz crytal with 16x PLL setting (80MHz)

  MAX_HZ   = 38_000


con { fixed io pins }

  RX1      = 31                                                 ' programming / terminal
  TX1      = 30

  SDA1     = 29                                                 ' eeprom / i2c
  SCL1     = 28


var

  long  cog                                                     ' cog running pwm code
  long  stack[16]                                               ' stack for Spin cog

  long  pwmtix                                                  ' system tix for pwm frequency
  long  resolution                                              ' value for 100%
  long  duty0                                                   ' duty cycle (0 to 100%)
  long  duty1

  long  dc0tix                                                  ' tix for pwm engine
  long  dc1tix


pub null

'' This is not an application


pub start(p0pin, p1pin, hz, maxval)

'' Start PWM object on pins p0 and p1
'' -- use -1 to disable either pin
'' -- hz is pwm frequency
'' -- maxval sets resolution (e.g, 255 for DMX-compatible pwm)

  stop                                                          ' stop if already running

  resolution := 10 #> maxval <# 1000                            ' set resolution

  freq(hz)                                                      ' set loop timing for pwm

  cog := cognew(dual_pwm(p0pin, p1pin), @stack) + 1             ' start pwm cog

  return cog


pub stop

'' Stops (unloads) pwm cog

  if (cog)
    cogstop(cog - 1)
    cog := 0

  longfill(@pwmtix, 0, 3)


pub freq(hz)

'' Sets pwm frequency
'' -- affects both outputs
'' -- best if outputs are off before changing frequency

  hz := 1 #> hz <# MAX_HZ                                       ' keep legal

  pwmtix := clkfreq / hz                                        ' calc new period tix
  set(0, duty0)                                                 ' update duty cycle tix
  set(1, duty1)

  return hz


pub set(ch, dc)

'' Set duty cycle (0 to resolution) for specific pwm channel (0 or 1)

  if (ch == 0)
    duty0 := 0 #> dc <# resolution
    dc0tix := -pwmtix * duty0 / resolution
    return duty0

  elseif (ch == 1)
    duty1 := 0 #> dc <# resolution
    dc1tix := -pwmtix * duty1 / resolution
    return duty1

  return -1                                                     ' bad channel


pri dual_pwm(p0, p1) | t

'' Run PWM outputs in background cog
'' -- sets p0 and p1 pins to outputs

  if (p0 => 0)
    ctra := (%00100 << 26) | p0                                 ' PWM / NCO
    frqa := 1                                                   ' setup frq for pwm
    dira[p0] := 1                                               ' make pin an output

  if (p1 => 0)
    ctrb := (%00100 << 26) | p1
    frqb := 1
    dira[p1] := 1

  t := cnt                                                      ' sync loop timing
  repeat                                                        ' run until stopped
    phsa := dc0tix
    phsb := dc1tix
    waitcnt(t += pwmtix)


con { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}